@@ -1,4 +1,4 @@
-/* GDM - The Gnome Display Manager
+/* GDM - The GNOME Display Manager
  * Copyright (C) 1998, 1999, 2000 Martin K, Petersen <mkp@mkp.net>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -55,6 +55,8 @@
 #include "gdmwm.h"
 #include "gdmcommon.h"
 
+static gchar *config_file;
+
 static gboolean RUNNING_UNDER_GDM = FALSE;
 
 enum {
@@ -88,7 +90,6 @@ static gint connection_type = 0;
 
 static void gdm_chooser_abort (const gchar *format, ...) G_GNUC_PRINTF (1, 2);
 static void gdm_chooser_warn (const gchar *format, ...) G_GNUC_PRINTF (1, 2);
-static void set_background (void);
 
 /* Exported for glade */
 void gdm_chooser_cancel (void);
@@ -171,8 +172,8 @@ static gchar *GdmMulticastAddr;
 /* */  gchar *GdmInfoMsgFile;
 /* */  gchar *GdmInfoMsgFont;
 /* */  gchar *GdmSoundProgram;
-/* */  gboolean GdmSoundOnLogin;
-/* */  gchar *GdmSoundOnLoginFile;
+/* */  gboolean GdmSoundOnLoginReady;
+/* */  gchar *GdmSoundOnLoginReadyFile;
 /* */  void
 /* */  gdm_kill_thingies (void)
 /* */  {
@@ -215,14 +216,6 @@ find_host_in_list (GdmChooserHost *host, GtkTreeIter *iter)
 	return FALSE;
 }
 
-static void
-setup_cursor (GdkCursorType type)
-{
-	GdkCursor *cursor = gdk_cursor_new (type);
-	gdk_window_set_cursor (gdk_get_default_root_window (), cursor);
-	gdk_cursor_unref (cursor);
-}
-
 static void
 gdm_chooser_host_dispose (GdmChooserHost *host)
 {
@@ -1389,8 +1382,8 @@ gdm_chooser_add_host (void)
 					 GTK_BUTTONS_OK,
 					 FALSE,
 					 _("Cannot find host"),
-					 _("I cannot find the host \"%s\", "
-					   "perhaps you have mistyped it."),
+					 _("Cannot find the host \"%s\". "
+					   "Perhaps you have mistyped it."),
 					 name);
 
 					if (RUNNING_UNDER_GDM)
@@ -1587,7 +1580,7 @@ gdm_chooser_parse_config (void)
 {
     VeConfig *cfg;
 
-    cfg = ve_config_get (GDM_CONFIG_FILE);
+    cfg = ve_config_get (config_file);
 
     GdmXineramaScreen = ve_config_get_int (cfg, GDM_KEY_XINERAMASCREEN);
     GdmGtkRC = ve_config_get_string (cfg, GDM_KEY_GTKRC);
@@ -1677,11 +1670,11 @@ display_chooser_information (void)
 		 GTK_BUTTONS_OK,
 		 _("The main area of this application shows the hosts on "
 		   "the local network that have \"XDMCP\" enabled. This "
-		   "allows users to login remotely to other machines as "
+		   "allows users to login remotely to other computers as "
 		   "if they were logged on using the console.\n\n"
 		   "You can rescan the network for new hosts by clicking "
 		   "\"Refresh\".  When you have selected a host click "
-		   "\"Connect\" to open a session to that machine."));
+		   "\"Connect\" to open a session to that computer."));
 	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
 
 	if (RUNNING_UNDER_GDM)
@@ -1875,7 +1868,7 @@ gdm_reread_config (int sig, gpointer data)
 	VeConfig *config;
 	/* reparse config stuff here.  At least ones we care about */
 
-	config = ve_config_get (GDM_CONFIG_FILE);
+	config = ve_config_get (config_file);
 
 	/* FIXME: The following is evil, we should update on the fly rather
 	 * then just restarting */
@@ -1893,8 +1886,7 @@ gdm_reread_config (int sig, gpointer data)
 	     ! bool_same (config, GdmDebug, GDM_KEY_DEBUG)) {
 		if (RUNNING_UNDER_GDM) {
 			/* Set busy cursor */
-			setup_cursor (GDK_WATCH);
-
+			gdm_common_setup_cursor (GDK_WATCH);
 			gdm_wm_save_wm_order ();
 		}
 
@@ -1907,7 +1899,10 @@ gdm_reread_config (int sig, gpointer data)
 	/* we only use the color and do it for all types except NONE */
 	if ( ! string_same (config, GdmBackgroundColor, GDM_KEY_BACKGROUNDCOLOR) ||
 	     ! int_same (config, GdmBackgroundType, GDM_KEY_BACKGROUNDTYPE)) {
-		set_background ();
+
+		if (GdmBackgroundType != GDM_BACKGROUND_NONE) {
+			setup_background_color (GdmBackgroundColor);
+		}
 	}
 
 	return TRUE;
@@ -1964,37 +1959,6 @@ struct poptOption xdm_options [] = {
 	{ NULL, 0, 0, NULL, 0}
 };
 
-static void
-set_background (void)
-{
-	if (GdmBackgroundType != GDM_BACKGROUND_NONE) {
-		GdkColor color;
-		GdkColormap *colormap;
-
-		if (ve_string_empty (GdmBackgroundColor) ||
-		    ! gdk_color_parse (GdmBackgroundColor, &color)) {
-			gdk_color_parse ("#007777", &color);
-		}
-
-		colormap = gdk_drawable_get_colormap
-			(gdk_get_default_root_window ());
-		/* paranoia */
-		if (colormap != NULL) {
-			gboolean success;
-			gdk_error_trap_push ();
-
-			gdk_colormap_alloc_colors (colormap, &color, 1,
-						   FALSE, TRUE, &success);
-
-			gdk_window_set_background (gdk_get_default_root_window (), &color);
-			gdk_window_clear (gdk_get_default_root_window ());
-
-			gdk_flush ();
-			gdk_error_trap_pop ();
-		}
-	}
-}
-
 static gboolean
 gdm_event (GSignalInvocationHint *ihint,
 	   guint		n_param_values,
@@ -2061,12 +2025,14 @@ main (int argc, char *argv[])
 	    exit (1);
     }
 
-    /* Should be a watch already, but just in case */
-    if (RUNNING_UNDER_GDM)
-	    setup_cursor (GDK_WATCH);
-
     glade_init();
 
+    config_file = gdm_common_get_config_file ();
+    if (config_file == NULL) {
+	    g_print (_("Could not access GDM configuration file.\n"));
+	    exit (EXIT_FAILURE);
+    }
+        
     gdm_chooser_parse_config();
 
     if (RUNNING_UNDER_GDM)
@@ -2074,6 +2040,16 @@ main (int argc, char *argv[])
 
     gdm_version = g_getenv ("GDM_VERSION");
 
+    /* Load the background as early as possible so GDM does not leave  */
+    /* the background unfilled.   The cursor should be a watch already */
+    /* but just in case */
+    if (RUNNING_UNDER_GDM) {
+	if (GdmBackgroundType != GDM_BACKGROUND_NONE)
+		setup_background_color (GdmBackgroundColor);
+
+	gdm_common_setup_cursor (GDK_WATCH);
+    }
+
     if (RUNNING_UNDER_GDM &&
 	gdm_version != NULL &&
 	strcmp (gdm_version, VERSION) != 0) {
@@ -2092,13 +2068,13 @@ main (int argc, char *argv[])
 					_("The chooser version (%s) does not match the daemon "
 					  "version (%s).  "
 					  "You have probably just upgraded gdm.  "
-					  "Please restart the gdm daemon or reboot the computer."),
+					  "Please restart the gdm daemon or the computer."),
 					VERSION, gdm_version);
 
 	    gtk_widget_show_all (dialog);
 	    gdm_wm_center_window (GTK_WINDOW (dialog));
 
-	    setup_cursor (GDK_LEFT_PTR);
+	    gdm_common_setup_cursor (GDK_LEFT_PTR);
 
 	    gtk_dialog_run (GTK_DIALOG (dialog));
 
@@ -2108,9 +2084,6 @@ main (int argc, char *argv[])
     gdm_chooser_gui_init();
     gdm_chooser_signals_init();
 
-    if (RUNNING_UNDER_GDM)
-	    set_background ();
-
     hosts = (char **)poptGetArgs (ctx);
     /* when no hosts on the command line, take them from the config */
     if (hosts == NULL ||
@@ -2124,15 +2097,13 @@ main (int argc, char *argv[])
     gdm_chooser_xdmcp_init (hosts);
     poptFreeContext (ctx);
 
-    if (RUNNING_UNDER_GDM) {
-	    guint sid = g_signal_lookup ("event",
-					 GTK_TYPE_WIDGET);
-	    g_signal_add_emission_hook (sid,
-					0 /* detail */,
-					gdm_event,
-					NULL /* data */,
-					NULL /* destroy_notify */);
-    }
+    guint sid = g_signal_lookup ("event",
+				 GTK_TYPE_WIDGET);
+    g_signal_add_emission_hook (sid,
+				0 /* detail */,
+				gdm_event,
+				NULL /* data */,
+				NULL /* destroy_notify */);
 
     gtk_widget_queue_resize (chooser);
     gtk_widget_show_now (chooser);
@@ -2158,7 +2129,7 @@ main (int argc, char *argv[])
 
     if (RUNNING_UNDER_GDM) {
 	    gdm_wm_restore_wm_order ();
-	    setup_cursor (GDK_LEFT_PTR);
+	    gdm_common_setup_cursor (GDK_LEFT_PTR);
     }
 
     gtk_main();
