@@ -59,6 +59,7 @@
 
 gboolean DOING_GDM_DEVELOPMENT = FALSE;
 static char *greeter_DefaultWelcome_key = GDM_KEY_DEFAULT_WELCOME;
+static char *greeter_DefaultWelcomeBacktest_key = GDM_KEY_DEFAULT_WELCOME_BACKTEST;
 static char *greeter_Welcome_key = GDM_KEY_WELCOME;
 
 GtkWidget *window;
@@ -104,6 +105,7 @@ gboolean GdmIncludeAll;
 gboolean GdmAllowRoot;
 gboolean GdmAllowRemoteRoot;
 static gboolean GdmDefaultWelcome;
+static gboolean GdmDefaultWelcomeBacktest;
 gchar *GdmWelcome;
 gchar *GdmServAuthDir;
 gchar *GdmInfoMsgFile;
@@ -116,12 +118,13 @@ gchar *GdmSoundOnLoginReadyFile;
 gchar *GdmSoundOnLoginSuccessFile;
 gchar *GdmSoundOnLoginFailureFile;
 gchar *GdmSoundProgram;
-
 gboolean GdmUseCirclesInEntry = FALSE;
 gboolean GdmUseInvisibleInEntry = FALSE;
 
+static gchar *config_file;
 static gboolean used_defaults = FALSE;
 gint greeter_current_delay = 0;
+static gboolean ignore_buttons = FALSE;
 
 /* FIXME: hack */
 GreeterItemInfo *welcome_string_info = NULL;
@@ -131,6 +134,12 @@ extern gboolean require_quarter;
 
 gboolean greeter_probably_login_prompt = FALSE;
 
+void
+greeter_ignore_buttons (gboolean val)
+{
+   ignore_buttons = val;
+}
+
 /* If in random theme mode then grab a random theme from those selected */
 static char *
 get_random_theme ()
@@ -141,7 +150,7 @@ get_random_theme ()
     int size;
 	int i;
 
-    VeConfig *config = ve_config_get (GDM_CONFIG_FILE);
+    VeConfig *config = ve_config_get (config_file);
 	themes_list = ve_config_get_string (config, GDM_KEY_GRAPHICAL_THEMES);
 
     if (ve_string_empty (themes_list))
@@ -168,22 +177,24 @@ greeter_parse_config (void)
 {
     VeConfig *config;
 
-    if (!g_file_test (GDM_CONFIG_FILE, G_FILE_TEST_EXISTS))
+    if (!g_file_test (config_file, G_FILE_TEST_EXISTS))
       {
 	syslog (LOG_ERR, _("%s: No configuration file: %s. Using defaults."), 
-		"greeter_parse_config", GDM_CONFIG_FILE);
+		"greeter_parse_config", config_file);
 	used_defaults = TRUE;
       }
 
     if (ve_string_empty (g_getenv ("GDM_IS_LOCAL"))) {
 	    greeter_Welcome_key = GDM_KEY_REMOTEWELCOME;
 	    greeter_DefaultWelcome_key = GDM_KEY_DEFAULT_REMOTEWELCOME;
+            greeter_DefaultWelcomeBacktest_key = GDM_KEY_DEFAULT_REMOTEWELCOME_BACKTEST;
     } else {
 	    greeter_Welcome_key = GDM_KEY_WELCOME;
 	    greeter_DefaultWelcome_key = GDM_KEY_DEFAULT_WELCOME;
+            greeter_DefaultWelcomeBacktest_key = GDM_KEY_DEFAULT_WELCOME_BACKTEST;
     }
 
-    config = ve_config_get (GDM_CONFIG_FILE);
+    config = ve_config_get (config_file);
 
 	GdmGraphicalThemeRand = ve_config_get_bool (config,
 	                                            GDM_KEY_GRAPHICAL_THEME_RAND);
@@ -239,6 +250,20 @@ greeter_parse_config (void)
     GdmWelcome = ve_config_get_translated_string (config, greeter_Welcome_key);
     GdmDefaultWelcome = ve_config_get_bool (config, greeter_DefaultWelcome_key);
 
+    /*
+     * For backwards compatibility.  If DefaultWelcome isn't in config file, then
+     * assume GdmDefaultWelcome is FALSE unless the string matches the default
+     */
+    GdmDefaultWelcomeBacktest = ve_config_get_bool (config, greeter_DefaultWelcomeBacktest_key);
+    if (GdmDefaultWelcomeBacktest == FALSE) {
+            if (strcmp (ve_sure_string (GdmWelcome), GDM_DEFAULT_WELCOME_MSG) == 0)
+                GdmDefaultWelcome == TRUE;
+            else if (strcmp (ve_sure_string (GdmWelcome), GDM_DEFAULT_REMOTEWELCOME_MSG) == 0)
+                GdmDefaultWelcome == TRUE;
+            else
+                GdmDefaultWelcome = FALSE;
+    }
+
     /* Replace default welcome message with one specified in the config file, if
      * use default is set to no */
     if (GdmDefaultWelcome && strcmp (greeter_Welcome_key, GDM_KEY_WELCOME) == 0) {
@@ -285,7 +310,7 @@ greeter_parse_config (void)
 
     GdmFlexiReapDelayMinutes = ve_config_get_int (config, GDM_KEY_FLEXI_REAP_DELAY_MINUTES);
 
-    GdmUse24Clock = ve_config_get_bool (config, GDM_KEY_USE_24_CLOCK);
+    GdmUse24Clock = gdm_common_select_time_format (config);
 
     GdmIconMaxWidth = ve_config_get_int (config, GDM_KEY_ICONWIDTH);
     GdmIconMaxHeight = ve_config_get_int (config, GDM_KEY_ICONHEIGHT);
@@ -370,11 +395,14 @@ greeter_ctrl_handler (GIOChannel *source,
 
 	tmp = ve_locale_to_utf8 (buf);
 	if (tmp != NULL && strcmp (tmp, _("Username:")) == 0) {
-		gdm_common_login_sound ();
+		gdm_common_login_sound (GdmSoundProgram,
+					GdmSoundOnLoginReadyFile,
+					GdmSoundOnLoginReady);
 		greeter_probably_login_prompt = TRUE;
 	} else {
 		greeter_probably_login_prompt = FALSE;
 	}
+	greeter_ignore_buttons (FALSE);
 	greeter_item_pam_prompt (tmp, PW_ENTRY_SIZE, TRUE);
 	g_free (tmp);
 	break;
@@ -384,6 +412,7 @@ greeter_ctrl_handler (GIOChannel *source,
 	buf[len-1] = '\0';
 
 	tmp = ve_locale_to_utf8 (buf);
+	greeter_ignore_buttons (FALSE);
 	greeter_item_pam_prompt (tmp, PW_ENTRY_SIZE, FALSE);
 	g_free (tmp);
 	break;
@@ -511,6 +540,7 @@ greeter_ctrl_handler (GIOChannel *source,
 
 	printf ("%c\n", STX);
 	fflush (stdout);
+	greeter_ignore_buttons (FALSE);
 	break;
 
     case GDM_QUIT:
@@ -679,24 +709,32 @@ static void
 greeter_action_ok (GreeterItemInfo *info,
                        gpointer         user_data)
 {
-   GreeterItemInfo *entry_info = greeter_lookup_id ("user-pw-entry");
-   if (entry_info && entry_info->item &&
-       GNOME_IS_CANVAS_WIDGET (entry_info->item) &&
-       GTK_IS_ENTRY (GNOME_CANVAS_WIDGET (entry_info->item)->widget))
+   if (ignore_buttons == FALSE)
      {
-       GtkWidget *entry;
-       entry = GNOME_CANVAS_WIDGET (entry_info->item)->widget;
-       greeter_item_pam_login (GTK_ENTRY (entry), entry_info);
-     }
+       GreeterItemInfo *entry_info = greeter_lookup_id ("user-pw-entry");
+       if (entry_info && entry_info->item &&
+           GNOME_IS_CANVAS_WIDGET (entry_info->item) &&
+           GTK_IS_ENTRY (GNOME_CANVAS_WIDGET (entry_info->item)->widget))
+         {
+           GtkWidget *entry;
+           entry = GNOME_CANVAS_WIDGET (entry_info->item)->widget;
+           greeter_ignore_buttons (TRUE);
+           greeter_item_pam_login (GTK_ENTRY (entry), entry_info);
+         }
+    }
 }
 
 static void
 greeter_action_cancel (GreeterItemInfo *info,
                        gpointer         user_data)
 {
-   greeter_item_ulist_disable ();
-   printf ("%c%c%c\n", STX, BEL, GDM_INTERRUPT_CANCEL);
-   fflush (stdout);
+   if (ignore_buttons == FALSE)
+     {
+       greeter_item_ulist_disable ();
+       greeter_ignore_buttons (TRUE);
+       printf ("%c%c%c\n", STX, BEL, GDM_INTERRUPT_CANCEL);
+       fflush (stdout);
+     }
 }
 
 static void
@@ -882,7 +920,7 @@ verify_gdm_version (void)
 static gboolean
 greeter_reread_config (int sig, gpointer data)
 {
-	VeConfig *config = ve_config_get (GDM_CONFIG_FILE);
+	VeConfig *config = ve_config_get (config_file);
 	char *theme, *theme_dir;
 	char *str;
 
@@ -949,11 +987,8 @@ greeter_reread_config (int sig, gpointer data)
 	GdmSoundOnLoginFailureFile = ve_config_get_string (config,
 	                             GDM_KEY_SOUND_ON_LOGIN_FAILURE_FILE);
 
-	if ( ! gdm_common_bool_same (config,
-				     GdmUse24Clock,
-				     GDM_KEY_USE_24_CLOCK)) {
-		GdmUse24Clock = ve_config_get_bool (config,
-						    GDM_KEY_USE_24_CLOCK);
+	if ( gdm_common_select_time_format (config) != GdmUse24Clock) {
+		GdmUse24Clock = gdm_common_select_time_format (config);
 		greeter_item_clock_update ();
 	}
 
@@ -963,6 +998,20 @@ greeter_reread_config (int sig, gpointer data)
 
 		GdmWelcome = ve_config_get_string (config, greeter_Welcome_key);
 		GdmDefaultWelcome = ve_config_get_bool (config, greeter_DefaultWelcome_key);
+                GdmDefaultWelcomeBacktest = ve_config_get_bool (config, greeter_DefaultWelcomeBacktest_key);
+
+                /*
+                 * For backwards compatibility.  If DefaultWelcome isn't in config file, then
+                 * assume GdmDefaultWelcome is FALSE unless the string matches the default
+                 */
+                if (GdmDefaultWelcomeBacktest == FALSE) {
+                        if (strcmp (ve_sure_string (GdmWelcome), GDM_DEFAULT_WELCOME_MSG) == 0)
+                                GdmDefaultWelcome == TRUE;
+                        else if (strcmp (ve_sure_string (GdmWelcome), GDM_DEFAULT_REMOTEWELCOME_MSG) == 0)
+                                GdmDefaultWelcome == TRUE;
+                        else
+                                GdmDefaultWelcome = FALSE;
+                }
 
 		if (GdmDefaultWelcome) {
 			if (strcmp (greeter_Welcome_key, GDM_KEY_WELCOME) == 0) {
@@ -1086,42 +1135,6 @@ get_theme_file (const char *in, char **theme_dir)
   return file;
 }
 
-/* Not to look too shaby on Xinerama setups */
-static void
-setup_background_color (void)
-{
-  GdkColormap *colormap;
-  GdkColor color;
-  VeConfig *config = ve_config_get (GDM_CONFIG_FILE);
-  char *bg_color = ve_config_get_string (config, GDM_KEY_BACKGROUNDCOLOR);
-
-  if (bg_color == NULL ||
-      bg_color[0] == '\0' ||
-      ! gdk_color_parse (bg_color, &color))
-    {
-      gdk_color_parse ("#007777", &color);
-    }
-
-  g_free (bg_color);
-
-  colormap = gdk_drawable_get_colormap
-	  (gdk_get_default_root_window ());
-  /* paranoia */
-  if (colormap != NULL)
-    {
-      gboolean success;
-      gdk_error_trap_push ();
-
-      gdk_colormap_alloc_colors (colormap, &color, 1,
-				 FALSE, TRUE, &success);
-      gdk_window_set_background (gdk_get_default_root_window (), &color);
-      gdk_window_clear (gdk_get_default_root_window ());
-
-      gdk_flush ();
-      gdk_error_trap_pop ();
-    }
-}
-
 /* The reaping stuff */
 static time_t last_reap_delay = 0;
 
@@ -1146,7 +1159,7 @@ reap_flexiserver (gpointer data)
 }
 
 
-void
+static void
 gdm_kill_thingies (void)
 {
 	/* Empty kill thingies */
@@ -1200,6 +1213,8 @@ gdm_event (GSignalInvocationHint *ihint,
 int
 main (int argc, char *argv[])
 {
+  VeConfig *config;
+  char *bg_color;
   struct sigaction hup;
   struct sigaction term;
   sigset_t mask;
@@ -1223,6 +1238,12 @@ main (int argc, char *argv[])
 
   setlocale (LC_ALL, "");
 
+  config_file = gdm_common_get_config_file ();
+  if (config_file == NULL) {
+	   g_print (_("Could not access GDM configuration file.\n"));
+	   exit (EXIT_FAILURE);
+  }
+
   greeter_parse_config ();
 
   gtk_init (&argc, &argv);
@@ -1247,6 +1268,12 @@ main (int argc, char *argv[])
   if (r != 0)
     return r;
 
+  /* Load the background as early as possible so GDM does not leave  */
+  /* the background unfilled.   The cursor should be a watch already */
+  /* but just in case */
+  config   = ve_config_get (config_file);
+  bg_color = ve_config_get_string (config, GDM_KEY_BACKGROUNDCOLOR);
+  setup_background_color (bg_color);
   greeter_session_init ();
 
   ve_signal_add (SIGHUP, greeter_reread_config, NULL);
@@ -1485,8 +1512,6 @@ main (int argc, char *argv[])
   gtk_window_move (GTK_WINDOW (window), gdm_wm_screen.x, gdm_wm_screen.y);
   gtk_widget_show_now (window);
 
-  setup_background_color ();
-
   /* can it ever happen that it'd be NULL here ??? */
   if G_UNLIKELY (window->window != NULL)
     {
@@ -1570,7 +1595,7 @@ main (int argc, char *argv[])
 				  _("The configuration was not found.  GDM is using "
 				    "defaults to run this session.  You should log in "
 				    "and create a configuration file with the GDM "
-				    "configuration program."));
+				    "configuration application."));
 
       gtk_widget_show_all (dialog);
       gdm_wm_center_window (GTK_WINDOW (dialog));
@@ -1627,7 +1652,7 @@ main (int argc, char *argv[])
 
   gdm_wm_restore_wm_order ();
 
-  gdm_common_show_info_msg ();
+  gdm_common_show_info_msg (GdmInfoMsgFile, GdmInfoMsgFont);
 
   gdm_common_setup_cursor (GDK_LEFT_PTR);
 
